-- Zombie Smasher 2
-- Hyperkani Oy 2012
-- Tero Pyylampi
--

-- Campaign 1 Level 1 (tutorial)

-- tutorial stage, stage 0 = starting texts to creating unit, 1 = spawning crate and, 2 = using weapon, 3 = normal game
local tutorialStage = 0
-- for tracking the tutorial states
local timePassed = 0.0
-- tag saved for strings
local stringTag = 0
-- enemy data
local enemiesDestroyed = 0
local enemiesSpawned = 0
-- enemy interval
local enemyInterval = 1.0
local enemyIntervalTimeLeft = 0.0
-- last spawn stage
local lastStage
-- victory flag
local victoryAchieved = false
-- game lost flag
local gameLost = false

-- initialize function which will be called after the game has been initialized
function init()
	-- lock camera
	setCameraLockState( true )
	-- set enemy speeds
	setSpeedMultipliers( 0.5, 0.6, 1.0, 1, 1, 1 )
end

-- update function which will be called constantly from the main game engine
function update( dt )
	-- first stage of the tutorial: show strings and spawn enemy
	if tutorialStage == 0 then
		-- welcome string after 1.5 seconds
		if timePassed <= 1.5 then
			timePassed = timePassed + dt
			if timePassed > 1.5 then
				showString( "Welcome to Zombie Smasher 2!", 3.0, 0.5, 0.5, 0.68, 34 )
				showString( "Your mission is to defend your village\nfrom ferocious zombies by all means!", 3.0, 0.5, 0.5, 0.42, 34 )
				timePassed = 1.6
			end
		-- initial zoom to the lower-right corner
		elseif timePassed <= 5.0 then
			timePassed = timePassed + dt
			if timePassed > 5.0 then
				setNewZoomLevelTarget( 22.0, 5.0, 1.9, 1.3, 3 )
				timePassed = 5.1
			end
		-- start slowly zooming away
		elseif timePassed <= 6.0 then
			timePassed = timePassed + dt
			if timePassed > 6.0 then
				setNewZoomLevelTarget( 22.0, 5.0, 1.8, 120, 0 )
				timePassed = 6.1
			end
		-- show string & spawn enemy
		elseif timePassed <= 6.5 then
			timePassed = timePassed + dt
			if timePassed > 6.5 then
				stringTag = showString( "Grab the appeared enemies with your finger\nand DRAG them hard against the mountain!", 999.0, 0.5, 0.5, 0.65, 34 )
				createNewEnemyUnit( 0, 0, 1 )
				createNewEnemyUnit( 0, 2, 1 )
				enemiesSpawned = enemiesSpawned + 2
				timePassed = 6.6
			end
		end

	-- enemy destroyed, show crate string and spawn crate
	elseif tutorialStage == 1 then
		if timePassed <= 7.5 then
			timePassed = timePassed + dt
			if timePassed > 7.5 then
				stringTag = showString( "Grab and destroy the spawned crate\nsimilarly against the mountain!", 999.0, 0.5, 0.5, 0.65, 34 )
				createTutorialCrate()
				timePassed = 7.6
			end
		end
	
	-- crate destroyed, use the weapon to destroy crate
	elseif tutorialStage == 2 then
		if timePassed <= 8.5 then
			timePassed = timePassed + dt
			if timePassed > 8.5 then
				stringTag = showString( "Use the received energy to smash\nthe spawned enemy with a rock!", 999.0, 0.5, 0.5, 0.65, 34 )
				disableWeaponButtons()
				enableSpecificWeaponButton( 4 )
				createNewEnemyUnit( 0, 1, 1 )
				enemiesSpawned = enemiesSpawned + 1
				timePassed = 8.6
			end
		end
	
	-- normal game mode, show level cleared
	elseif tutorialStage == 3 then
		if timePassed <= 10.0 then
			timePassed = timePassed + dt
			if timePassed > 10.0 then
				showString( "You are now ready to play!\nGood luck!", 3.0, 0.5, 0.5, 0.65, 34 )
				createNewEnemyUnit( 1, 0, 1 )
				enemiesSpawned = enemiesSpawned + 1
				timePassed = 10.1
			end
		end
		-- zooming back to the starting pos AFTER the game has been finished
		if timePassed >= 100.0 then
			timePassed = timePassed + dt
			if timePassed >= 100.5 then
				setNewZoomLevelTarget( 0.0, 15, 1.0, 0.5, 2 )
				timePassed = 99.0
			end
		elseif timePassed >= 99.0 then
			timePassed = timePassed + dt
			if timePassed >= 99.5 then
				if victoryAchieved then 
					showLevelCleared()
				else
					showLevelFailed()
				end
				timePassed = 90.0
			end
		end
	end
	
	-- spawn rest of the enemies
	if enemiesDestroyed >= 3 and tutorialStage == 3 and gameLost == false then
		-- decrease time
		enemyIntervalTimeLeft = enemyIntervalTimeLeft - dt
		if enemyIntervalTimeLeft <= 0 then
			-- spawn enemy and increase counter
			if enemiesSpawned == 3 then
				enemyInterval = 1.3
				createNewEnemyUnit( 1, 2, 1 )
				enemiesSpawned = enemiesSpawned + 1
				lastStage = 2
				showString( "Now destroy the rest of the enemies!", 5.0, 0.5, 0.5, 0.75, 34 )
				setNewZoomLevelTarget( 22.0, 5.0, 1.8, 120, 0 )
			elseif enemiesSpawned < 20 then
				stage = math.random( 0, 2 )
				if stage == lastStage then
					stage = ( stage + 1 ) % 3
				end
				lastStage = stage
				createNewEnemyUnit( 1, math.random( 0, 2 ), 1 )
				enemiesSpawned = enemiesSpawned + 1
			elseif enemiesSpawned < 60 then
				enemyInterval = 1.8
				e1 = math.random( 0, 2 )
				e2 = ( e1 + 1 ) % 3
				createNewEnemyUnit( 1, e1, 1 )
				createNewEnemyUnit( 1, e2, 1 )
				enemiesSpawned = enemiesSpawned + 2
			end
			-- reset interval
			--enemyInterval = enemyInterval * 0.97
			enemyIntervalTimeLeft = enemyInterval
		end
	end
	
	-- check for victory conditions
	if gameLost == false and victoryAchieved == false and enemiesSpawned >= 60 and enemiesLeftOnField() == 0 then
		victoryAchieved = true
		setNewZoomLevelTarget( 9.0, 11.0, 1.4, 0.5, 1 )
		timePassed = 100.0
	end
end

-- called when a single enemy reaches the base
function enemyReachedBase( healthLeft )
	enemiesDestroyed = enemiesDestroyed + 1
	if healthLeft == 0 and gameLost == false then
		gameLost = true
		setNewZoomLevelTarget( 9.0, 11.0, 1.4, 0.5, 1 )
		timePassed = 100.0
	end
end

-- called when a single enemy has been destroyed
function enemyDestroyed( enemyType )
	-- hide text
	if enemiesDestroyed == 1 then
		hideString( stringTag, false )
		enemyIntervalTimeLeft = 0.0
		tutorialStage = 1
	-- check for interval
	elseif enemiesLeftOnField() == 0 then
		enemyIntervalTimeLeft = 0
	end
	-- increase counter
	enemiesDestroyed = enemiesDestroyed + 1
end

-- called when a single weapon has been launched
function weaponLaunched( weaponType )
	if weaponType == 4 and tutorialStage == 2 then
		hideString( stringTag, false )
		tutorialStage = 3
	end
end

-- calculates the amount of stars to be awarded for completing this level
function calculateStars( scoreCollected, healthLeft )
	if scoreCollected >= ( enemiesSpawned * maxPointsPerEnemy() * 0.65 ) and healthLeft > 3 then
		return 3
	elseif scoreCollected >= ( enemiesSpawned * maxPointsPerEnemy() * 0.35 ) and healthLeft > 1 then
		return 2
	else
		return 1
	end
end


-- for reporting main engine the amount of enemies destroyed during this level
function enemiesDestroyedInLevel()
	return enemiesDestroyed
end

-- for reporting main engine how many enemies there are to come during this level
function enemiesTotalInLevel()
	return 60
end

function tutorialCrateDestroyed()
	tutorialStage = 2
	hideString( stringTag, false )
	disableWeaponButtons()
end	