-- Zombie Smasher 2
-- Hyperkani Oy 2012
-- Tero Pyylampi
--

-- Campaign 1 Level 10 (free camera and boss type enemy)

-- for tracking the tutorial states
local timePassed = 0.0
-- enemy data
local enemiesDestroyed = 0
local enemiesSpawned = 0
-- enemy interval
local enemyInterval = 2.0
local enemyIntervalTimeLeft = 1.3
-- victory flag
local victoryAchieved = false
local bossDestroyed = false
-- game lost flag
local gameLost = false
-- level stages, 0 = initial stage and complete level, 1 = zooming out as the player has lost or won the game
local levelStage = 0

-- initialize function which will be called after the game has been initialized
function init()
	-- lock camera
	setCameraLockState( true )
	-- set enemy speeds
	setSpeedMultipliers( 0.5, 0.9, 1., 1, 1, 10 )
end

-- update function which will be called constantly from the main game engine
function update( dt )
	-- starting stage
	if levelStage == 0 then
		-- welcome string after 1.5 seconds
		if timePassed <= 1.0 then
			timePassed = timePassed + dt
			if timePassed > 1.0 then
				showString( "Level 10", 2.5, 0.5, 0.5, 0.65, 34 )
				showString( "(Boss)", 2.5, 0.5, 0.5, 0.5, 26 )
			end
		-- initial zoom to the center and then releasing the camera
		elseif timePassed <= 3.0 then
			timePassed = timePassed + dt
			if timePassed > 3.0 then
				setNewZoomLevelTarget( 0.0, 16.0, 1.2, 1.0, 3 )
			end
		elseif levelStage == 0 and timePassed <= 4.0 then
			timePassed = timePassed + dt
		-- camera playing a bit
		elseif levelStage == 0 and timePassed > 4.0 then
			timePassed = timePassed + dt
			if( timePassed >= 34.0 ) then
				timePassed = 5.0
				setNewZoomLevelTarget( -3.0 + math.random( 0, 6 ), 14.0 + math.random( 0, 3 ), 1.1 + ( math.random( 0, 3 ) / 10.0 ), 30.0, 0 )
			end
		end
	
	-- showing the end string after the zoom out event (either lost or won)
	elseif levelStage == 1 then
		if timePassed <= 1.0 then
			timePassed = timePassed + dt
			if timePassed > 1.0 then
				if victoryAchieved then 
					showLevelCleared()
				else
					showLevelFailed()
				end
			end
		end
	end
	
	-- spawn enemies
	if levelStage == 0 and gameLost == false and victoryAchieved == false and bossDestroyed == false then
		enemyIntervalTimeLeft = enemyIntervalTimeLeft - dt
		if enemiesSpawned == 0 and enemyIntervalTimeLeft <= 0 then
			createNewEnemyUnit( 3, 1, 0 )
			enemiesSpawned = enemiesSpawned + 1
			enemyIntervalTimeLeft = enemyInterval
		elseif enemyIntervalTimeLeft <= 0 then
			createNewEnemyUnit( 1, math.random( 0, 2 ), math.random( 0, 1 ) )
			enemiesSpawned = enemiesSpawned + 1
			enemyInterval = enemyInterval * 0.99
			enemyIntervalTimeLeft = enemyInterval
		end
	end
	
	-- check for victory conditions
	if gameLost == false and victoryAchieved == false and bossDestroyed == true then
		victoryAchieved = true
		setNewZoomLevelTarget( 0.0, 15.0, 1.0, 1.0, 3 )
		levelStage = 1
		timePassed = 0
		setCameraLockState( true )
	end
end

-- called when a single enemy reaches the base
function enemyReachedBase( healthLeft )
	enemiesDestroyed = enemiesDestroyed + 1
	if healthLeft == 0 and gameLost == false and victoryAchieved == false then
		gameLost = true
		setNewZoomLevelTarget( 0.0, 15.0, 1.0, 1.0, 3 )
		setCameraLockState( true )
		timePassed = 0.0
		levelStage = 1
	end
end

-- called when a single enemy has been destroyed
function enemyDestroyed( enemyType )
	-- increase counter
	enemiesDestroyed = enemiesDestroyed + 1
	if enemyType == 3 then
		bossDestroyed = true
	end
end

-- called when a single weapon has been launched
function weaponLaunched( weaponType )
	--
end

-- calculates the amount of stars to be awarded for completing this level
function calculateStars( scoreCollected, healthLeft )
	if healthLeft > 4 then
		return 3
	elseif healthLeft > 2 then
		return 2
	else
		return 1
	end
end

-- for reporting main engine the amount of enemies destroyed during this level
function enemiesDestroyedInLevel()
	return enemiesDestroyed
end

-- for reporting main engine how many enemies there are to come during this level
function enemiesTotalInLevel()
	return -1
end