-- Zombie Smasher 2
-- Hyperkani Oy 2012
-- Tero Pyylampi
--

-- Campaign 2 Level 1

-- for tracking the tutorial states
local timePassed = 0.0
-- enemy data
local enemiesDestroyed = 0
local enemiesSpawned = 0
-- enemy interval
local enemyInterval = 1.0
local enemyIntervalTimeLeft = 0
-- victory flag
local victoryAchieved = false
-- game lost flag
local gameLost = false
-- level stages, 0 = center of the village and zooming away a while, 1 = moving slowly to the lower left corner, 2 = lower right corner and zooming away, 3 = zooming slowly to the center, 4 = center
local levelStage = 0

-- initialize function which will be called after the game has been initialized
function init()
	-- lock camera
	setCameraLockState( true )
	-- set enemy speeds
	setSpeedMultipliers( 1.0, 1.05, 1.1, 1, 1.05, 1 )
end

-- update function which will be called constantly from the main game engine
function update( dt )
	-- starting stage
	if levelStage == 0 then
		-- welcome string after 1.5 seconds
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if timePassed >= 1.0 then
				showString( "Level 12", 2.5, 0.5, 0.5, 0.65, 34 )
			end
		-- initial zoom to the center of the village
		elseif timePassed < 3.0 then
			timePassed = timePassed + dt
			if( timePassed >= 3.0 ) then
				setNewZoomLevelTarget( 0, 17.0, 1.4, 1.0, 3 )
				timePassed = 3.0
			end
		-- slowly zooming away
		elseif timePassed < 4.5 then
			timePassed = timePassed + dt
			if timePassed >= 4.5 then
				setNewZoomLevelTarget( 0, 17.0, 1.3, 100, 0 )
				timePassed = 4.5
			end
		end
	
	-- stage 1 = move slowly to the left corner
	elseif levelStage == 1 then
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if timePassed >= 1.0 then
				setNewZoomLevelTarget( -22.0, 5.0, 1.9, 45, 0 )
				timePassed = 1.0
			end
		end

	-- stage 2 = move to the lower right corner and slowly zoom away
	elseif levelStage == 2 then
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if( timePassed >= 1.0 ) then
				setNewZoomLevelTarget( 22.0, 5.0, 1.9, 1.0, 3 )
				timePassed = 1.0
			end
		elseif timePassed <= 2.5 then
			timePassed = timePassed + dt
			if( timePassed > 2.5 ) then
				setNewZoomLevelTarget( 22.0, 5.0, 1.7, 60, 0 )
			end
		end
		
	-- stage 3 = moving back to the center
	elseif levelStage == 3 then
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if( timePassed >= 1.0 ) then
				setNewZoomLevelTarget( 0, 17.0, 1.4, 30.0, 0 )
				timePassed = 1.0
			end
		end
			
	-- stage 3 = zooming out a bit at the center
	elseif levelStage == 4 then
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if( timePassed >= 1.0 ) then
				setNewZoomLevelTarget( 0, 17.0, 1.3, 60.0, 0 )
				timePassed = 1.0
			end
		end
	
	-- zooming back to the starting pos AFTER the game has been finished (either lost or won)
	elseif levelStage == 5 then
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if timePassed >= 1.0 then
				if victoryAchieved then 
					showLevelCleared()
				else
					showLevelFailed()
				end
			end
		end
	end
	
	-- spawn enemies
	if levelStage < 5 and gameLost == false then
		enemyIntervalTimeLeft = enemyIntervalTimeLeft - dt
		if enemyIntervalTimeLeft < 0 then
			-- spawn new enemy on both sides
			if levelStage == 0 and enemiesSpawned < 80 then
				e1 = math.random( 1, 3 )
				if e1 == 3 then e1 = 4 end
				e2 = math.random( 1, 3 )
				if e2 == 3 then e2 = 4 end
				createNewEnemyUnit( e1, math.random( 0, 2 ), 0 )
				createNewEnemyUnit( e2, math.random( 0, 2 ), 1 )
				enemiesSpawned = enemiesSpawned + 2
			-- moving to the left, only spawn there
			elseif levelStage == 1 and enemiesSpawned < 160 then
				e1 = math.random( 0, 2 )
				e2 = ( e1 + 1 ) % 3
				createNewEnemyUnit( math.random( 1, 2 ), e1, 0 )
				createNewEnemyUnit( 4, e2, 0 )
				enemiesSpawned = enemiesSpawned + 2
			-- moving to the right, only spawn there
			elseif levelStage == 2 and enemiesSpawned < 250 then
				e1 = math.random( 0, 2 )
				e2 = ( e1 + 1 ) % 3
				e3 = ( e2 + 1 ) % 3
				createNewEnemyUnit( 4, e1, 1 )
				createNewEnemyUnit( 1, e2, 1 )
				s2 = math.random( 1, 3 )
				if s2 == 3 then s2 = 4 end
				createNewEnemyUnit( s2, e3, 1 )
				enemiesSpawned = enemiesSpawned + 3
			-- moving back to center
			elseif levelStage == 3 and enemiesSpawned < 310 then
				e1 = math.random( 0, 2 )
				e2 = ( e1 + 1 ) % 3
				e3 = ( e2 + 1 ) % 3
				createNewEnemyUnit( 4, e1, 1 )
				createNewEnemyUnit( 1, e2, 1 )
				s2 = math.random( 1, 3 )
				if s2 == 3 then s2 = 4 end
				createNewEnemyUnit( s2, e3, 1 )
				enemiesSpawned = enemiesSpawned + 3
			-- finals
			elseif levelStage == 4 and enemiesSpawned < 390 then
				e1 = math.random( 0, 2 )
				e2 = ( e1 + 1 ) % 3
				e3 = ( e2 + 1 ) % 3
				e4 = ( e3 + 1 ) % 3
				createNewEnemyUnit( 4, e1, 0 )
				createNewEnemyUnit( math.random( 1, 2 ), e2, 0 )
				createNewEnemyUnit( 4, e3, 1 )
				createNewEnemyUnit( math.random( 1, 2 ), e4, 1 )
				enemiesSpawned = enemiesSpawned + 4
			end
			-- reset timer
			--enemyInterval = enemyInterval * 0.99
			enemyIntervalTimeLeft = enemyInterval
		end
	end
	
	-- advance in stages
	if levelStage == 0 and enemiesSpawned >= 80 and enemiesLeftOnField() == 0 then
		levelStage = 1
		timePassed = 0
	elseif levelStage == 1 and enemiesSpawned >= 160 and enemiesLeftOnField() == 0 then
		levelStage = 2
		timePassed = 0
	elseif levelStage == 2 and enemiesSpawned >= 250 and enemiesLeftOnField() == 0 then
		levelStage = 3
		timePassed = 0
	elseif levelStage == 3 and enemiesSpawned >= 310 and enemiesLeftOnField() == 0 then
		levelStage = 4
		timePassed = 0
	-- check for victory conditions
	elseif gameLost == false and victoryAchieved == false and levelStage == 4 and enemiesSpawned >= 390 and enemiesLeftOnField() == 0 then
		victoryAchieved = true
		setNewZoomLevelTarget( 0.0, 15.0, 1.0, 1.0, 3 )
		levelStage = 5
		timePassed = 0
	end
end

-- called when a single enemy reaches the base
function enemyReachedBase( healthLeft )
	enemiesDestroyed = enemiesDestroyed + 1
	if healthLeft == 0 and gameLost == false then
		gameLost = true
		setNewZoomLevelTarget( 0.0, 15.0, 1.0, 1.0, 3 )
		timePassed = 0.0
		levelStage = 5
	end
end

-- called when a single enemy has been destroyed
function enemyDestroyed( enemyType )
	-- increase counter
	enemiesDestroyed = enemiesDestroyed + 1
end

-- called when a single weapon has been launched
function weaponLaunched( weaponType )
	--
end

-- calculates the amount of stars to be awarded for completing this level
function calculateStars( scoreCollected, healthLeft )
	if scoreCollected >= ( enemiesSpawned * maxPointsPerEnemy() * 0.72 ) and healthLeft > 4 then
		return 3
	elseif scoreCollected >= ( enemiesSpawned * maxPointsPerEnemy() * 0.46 ) and healthLeft > 2 then
		return 2
	else
		return 1
	end
end

-- for reporting main engine the amount of enemies destroyed during this level
function enemiesDestroyedInLevel()
	return enemiesDestroyed
end

-- for reporting main engine how many enemies there are to come during this level
function enemiesTotalInLevel()
	return 390
end
