-- Zombie Smasher 2
-- Hyperkani Oy 2012
-- Tero Pyylampi
--

-- Campaign 2 Level 4 (introducing the small bird enemy)

-- for tracking the states
local timePassed = 0.0
-- enemy data
local enemiesDestroyed = 0
local enemiesSpawned = 0
-- enemy interval
local enemyInterval = 0.7
local enemyIntervalTimeLeft = 4.0
-- victory flag
local victoryAchieved = false
-- game lost flag
local gameLost = false
-- level stages, 0 = upper right corner, 1 = zooming back slowly all the time, 2 = zooming away
local levelStage = 0

-- initialize function which will be called after the game has been initialized
function init()
	-- lock camera
	setCameraLockState( true )
	-- set enemy speeds
	setSpeedMultipliers( 0.5, 1.15, 1.15, 1, 1.1, 1.0 )
end

-- update function which will be called constantly from the main game engine
function update( dt )
	-- starting stage
	if levelStage == 0 then
		-- welcome string after 1.5 seconds
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if timePassed > 1.0 then
				showString( "Level 14", 2.5, 0.5, 0.5, 0.65, 34 )
				timePassed = 1.0
			end
		-- initial zoom to the lower-right corner
		elseif timePassed < 3.0 then
			timePassed = timePassed + dt
			if timePassed > 3.0 then
				setNewZoomLevelTarget( 20.0, 22.0, 1.8, 1.0, 3 )
				timePassed = 3.0
			end
		-- zoom away
		elseif timePassed < 4.5 then
			timePassed = timePassed + dt
			if timePassed > 4.5 then
				setNewZoomLevelTarget( 11.0, 17.0, 1.4, 60.0, 0 )
				timePassed = 4.5
			end
		end
	
	-- zooming back to the starting pos AFTER the game has been finished (either lost or won)
	elseif levelStage == 2 then
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if timePassed >= 1.0 then
				if victoryAchieved then 
					showLevelCleared()
				else
					showLevelFailed()
				end
			end
		end
	end
	
	-- spawn enemies
	if levelStage ~= 2 and gameLost == false then
		enemyIntervalTimeLeft = enemyIntervalTimeLeft - dt
		if enemyIntervalTimeLeft < 0 then
			-- spawn new enemy
			if levelStage == 0 and enemiesSpawned < 50 then
				enemyIntervalTimeLeft = enemyInterval
				if enemiesSpawned == 0 then
					createNewEnemyUnit( 5, 0, 1 )
					enemyIntervalTimeLeft = enemyInterval * 4.0
				elseif enemiesSpawned == 1 then
					createNewEnemyUnit( 5, 2, 1 )
					enemyIntervalTimeLeft = enemyInterval * 2.0
				else
					createNewEnemyUnit( 5, math.random( 0, 2 ), 1 )
				end
				enemiesSpawned = enemiesSpawned + 1
				-- reset timer
				--enemyInterval = enemyInterval * 0.98
			elseif levelStage == 1 and enemiesSpawned < 230 then
				enemyInterval = 1.2
				e1 = math.random( 0, 2 )
				e2 = ( e1 + 1 ) % 3
				e3 = math.random( 0, 2 )
				createNewEnemyUnit( 1, e1, 1 ) 
				createNewEnemyUnit( 4, e2, 1 ) 
				createNewEnemyUnit( 5, e3, 1 ) 
				-- reset timer
				--enemyInterval = enemyInterval * 0.995
				enemyIntervalTimeLeft = enemyInterval
				enemiesSpawned = enemiesSpawned + 3
			end
		end
	end
	
	-- advance in stages
	if levelStage == 0 and enemiesSpawned >= 50 then
		levelStage = 1
		timePassed = 0
	-- check for victory conditions
	elseif gameLost == false and victoryAchieved == false and levelStage == 1 and enemiesSpawned >= 230 and enemiesLeftOnField() == 0 then
		victoryAchieved = true
		setNewZoomLevelTarget( 0.0, 15.0, 1.0, 1.0, 3 )
		levelStage = 2
		timePassed = 0
	end
end

-- called when a single enemy reaches the base
function enemyReachedBase( healthLeft )
	enemiesDestroyed = enemiesDestroyed + 1
	if healthLeft == 0 and gameLost == false then
		gameLost = true
		setNewZoomLevelTarget( 0.0, 15.0, 1.0, 1.0, 3 )
		timePassed = 0.0
		levelStage = 2
	end
end

-- called when a single enemy has been destroyed
function enemyDestroyed( enemyType )
	-- increase counter
	enemiesDestroyed = enemiesDestroyed + 1
end

-- called when a single weapon has been launched
function weaponLaunched( weaponType )
	--
end

-- calculates the amount of stars to be awarded for completing this level
function calculateStars( scoreCollected, healthLeft )
	if scoreCollected >= ( enemiesSpawned * maxPointsPerEnemy() * 0.76 ) and healthLeft > 3 then
		return 3
	elseif scoreCollected >= ( enemiesSpawned * maxPointsPerEnemy() * 0.45 ) and healthLeft > 2 then
		return 2
	else
		return 1
	end
end

-- for reporting main engine the amount of enemies destroyed during this level
function enemiesDestroyedInLevel()
	return enemiesDestroyed
end

-- for reporting main engine how many enemies there are to come during this level
function enemiesTotalInLevel()
	return 230
end
