-- Zombie Smasher 2
-- Hyperkani Oy 2012
-- Tero Pyylampi
--

-- Campaign 2 Level 5 (introducing the small bird enemy)

-- for tracking the states
local timePassed = 0.0
-- enemy data
local enemiesDestroyed = 0
local enemiesSpawned = 0
-- enemy interval
local enemyInterval = 1.3
local enemyIntervalTimeLeft = 4.0
-- victory flag
local victoryAchieved = false
-- game lost flag
local gameLost = false
-- level stages, 0 = middle of the village, 1 = lower left corner, 2 = middle again, 3 = zooming away
local levelStage = 0

-- initialize function which will be called after the game has been initialized
function init()
	-- lock camera
	setCameraLockState( true )
	-- set enemy speeds
	setSpeedMultipliers( 0.5, 1.20, 1.20, 1, 1.15, 1.05 )
end

-- update function which will be called constantly from the main game engine
function update( dt )
	-- starting stage
	if levelStage == 0 then
		-- welcome string after 1.5 seconds
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if timePassed >= 1.0 then
				showString( "Level 15", 2.5, 0.5, 0.5, 0.65, 34 )
				timePassed = 1.0
			end
		-- initial zoom to the middle of the screen
		elseif timePassed < 3.0 then
			timePassed = timePassed + dt
			if timePassed >= 3.0 then
				setNewZoomLevelTarget( 0.0, 17.0, 1.2, 1.0, 3 )
				timePassed = 3.0
			end
		-- zoom in
		elseif timePassed < 4.5 then
			timePassed = timePassed + dt
			if timePassed >= 4.5 then
				setNewZoomLevelTarget( 0.0, 17.0, 1.3, 60.0, 0 )
				timePassed = 4.5
			end
		end
	-- zooming to the lower left corner
	elseif levelStage == 1 then
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if timePassed >= 1.0 then
				setNewZoomLevelTarget( -22.0, 5.0, 2.0, 1.0, 3 )
				timePassed = 1.0
			end
		elseif timePassed < 2.5 then
			timePassed = timePassed + dt
			if timePassed >= 2.5 then
				setNewZoomLevelTarget( -22.0, 5.0, 1.8, 60.0, 3 )
				timePassed = 1.0
			end
		end
	-- back to the middle again
	elseif levelStage == 2  then
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if timePassed >= 1.0 then
				setNewZoomLevelTarget( 0.0, 17.0, 1.2, 1.0, 3 )
				timePassed = 1.0
			end
		-- zoom in
		elseif timePassed < 2.5 then
			timePassed = timePassed + dt
			if timePassed >= 2.5 then
				setNewZoomLevelTarget( 0.0, 17.0, 1.3, 60.0, 0 )
				timePassed = 2.5
			end
		end
	-- zooming back to the starting pos AFTER the game has been finished (either lost or won)
	elseif levelStage == 3 then
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if timePassed >= 1.0 then
				if victoryAchieved then 
					showLevelCleared()
				else
					showLevelFailed()
				end
			end
		end
	end
	
	-- spawn enemies
	if levelStage < 3 and gameLost == false then
		enemyIntervalTimeLeft = enemyIntervalTimeLeft - dt
		if enemyIntervalTimeLeft < 0 then
			-- spawn new enemy
			if levelStage == 0 and enemiesSpawned < 90 then
				createNewEnemyUnit( 5, math.random( 0, 2 ), math.random( 0, 1 ) )
				e1 = math.random( 1, 3 )
				if e1 == 3 then e1 = 4 end
				e2 = math.random( 1, 3 )
				if e2 == 3 then e2 = 4 end
				createNewEnemyUnit( e1, math.random( 0, 2 ), 0 )
				createNewEnemyUnit( e2, math.random( 0, 2 ), 1 )
				enemiesSpawned = enemiesSpawned + 3
			elseif levelStage == 1 and enemiesSpawned < 180 then
				e1 = math.random( 1, 3 )
				if e1 == 3 then e1 = 4 end
				e2 = math.random( 1, 3 )
				if e2 == 3 then e2 = 4 end
				e3 = math.random( 1, 3 )
				if e3 == 3 then e3 = 4 end
				createNewEnemyUnit( e1, 0, 0 )
				createNewEnemyUnit( e2, 1, 0 )
				createNewEnemyUnit( e3, 2, 0 )
				enemiesSpawned = enemiesSpawned + 3
			elseif levelStage == 2 and enemiesSpawned < 400 then
				createNewEnemyUnit( 5, math.random( 0, 2 ), 0 )
				createNewEnemyUnit( 5, math.random( 0, 2 ), 1 )
				e1 = math.random( 1, 3 )
				if e1 == 3 then e1 = 4 end
				e2 = math.random( 1, 3 )
				if e2 == 3 then e2 = 4 end
				createNewEnemyUnit( e1, math.random( 0, 2 ), 0 )
				createNewEnemyUnit( e2, math.random( 0, 2 ), 1 )
				enemiesSpawned = enemiesSpawned + 4
			end
			enemyIntervalTimeLeft = enemyInterval
		end
	end
	
	-- advance in stages
	if levelStage == 0 and enemiesSpawned >= 90 and enemiesLeftOnField() == 0 then
		levelStage = 1
		timePassed = 0
	elseif levelStage == 1 and enemiesSpawned >= 180 and enemiesLeftOnField() == 0 then
		enemyInterval = enemyInterval + 0.4
		levelStage = 2
		timePassed = 0
	-- check for victory conditions
	elseif gameLost == false and victoryAchieved == false and levelStage == 2 and enemiesSpawned >= 400 and enemiesLeftOnField() == 0 then
		victoryAchieved = true
		setNewZoomLevelTarget( 0.0, 15.0, 1.0, 1.0, 3 )
		levelStage = 3
		timePassed = 0
	end
end

-- called when a single enemy reaches the base
function enemyReachedBase( healthLeft )
	enemiesDestroyed = enemiesDestroyed + 1
	if healthLeft == 0  and gameLost == false then
		gameLost = true
		setNewZoomLevelTarget( 0.0, 15.0, 1.0, 1.0, 3 )
		timePassed = 0.0
		levelStage = 3
	end
end

-- called when a single enemy has been destroyed
function enemyDestroyed( enemyType )
	-- increase counter
	enemiesDestroyed = enemiesDestroyed + 1
end

-- called when a single weapon has been launched
function weaponLaunched( weaponType )
	--
end

-- calculates the amount of stars to be awarded for completing this level
function calculateStars( scoreCollected, healthLeft )
	if scoreCollected >= ( enemiesSpawned * maxPointsPerEnemy() * 0.76 ) and healthLeft > 3 then
		return 3
	elseif scoreCollected >= ( enemiesSpawned * maxPointsPerEnemy() * 0.45 ) and healthLeft > 2 then
		return 2
	else
		return 1
	end
end

-- for reporting main engine the amount of enemies destroyed during this level
function enemiesDestroyedInLevel()
	return enemiesDestroyed
end

-- for reporting main engine how many enemies there are to come during this level
function enemiesTotalInLevel()
	return 400
end
