-- Zombie Smasher 2
-- Hyperkani Oy 2012
-- Tero Pyylampi
--

-- Campaign 1 Level 7 (again playing with camera, possibly introducing a new enemy?)

-- for tracking the tutorial states
local timePassed = 0.0
-- enemy data
local enemiesDestroyed = 0
local enemiesSpawned = 0
-- enemy interval
local enemyInterval = 1.2
local enemyIntervalTimeLeft = 0
-- victory flag
local victoryAchieved = false
-- game lost flag
local gameLost = false
-- level stages, 0 = left corner, 1 = moving to the center, 2 = moving to the right corner, 3 = fast back to center and zooming slowly away, 4 = end
local levelStage = 0

-- initialize function which will be called after the game has been initialized
function init()
	-- lock camera
	setCameraLockState( true )
	-- set enemy speeds
	setSpeedMultipliers( 0.5, 1.25, 1.25, 1, 1.20, 1.15 )
end

-- update function which will be called constantly from the main game engine
function update( dt )
	-- starting stage
	if levelStage == 0 then
		-- welcome string after 1 seconds
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if timePassed >= 1.0 then
				showString( "Level 17", 2.5, 0.5, 0.5, 0.65, 34 )
				timePassed = 1.0
			end
		-- initial zoom to the left corner
		elseif timePassed < 3.0 then
			timePassed = timePassed + dt
			if timePassed >= 3.0 then
				setNewZoomLevelTarget( -11.0, 10, 1.5, 0.5, 1 )
				timePassed = 3.0
			end
		elseif timePassed < 3.5 then
			timePassed = timePassed + dt
			if( timePassed >= 3.5 ) then
				setNewZoomLevelTarget( -22, 5.0, 2.0, 0.5, 2 )
				timePassed = 3.5
			end
		-- slowly zooming away
		elseif timePassed < 4.5 then
			timePassed = timePassed + dt
			if timePassed >= 4.5 then
				setNewZoomLevelTarget( -22, 5.0, 1.7, 70, 0 )
				timePassed = 4.5
			end
		end
	
	-- stage 1 = move slowly to the center of the village
	elseif levelStage == 1 then
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if timePassed >= 1.0 then
				setNewZoomLevelTarget( 0.0, 17.0, 1.2, 30, 0 )
				timePassed = 1.0
			end
		end

	-- stage 2 = move slowly to the right corner
	elseif levelStage == 2 then
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if timePassed >= 1.0 then
				setNewZoomLevelTarget( 22.0, 5.0, 2.0, 30, 0 )
			end
		end
		
	-- stage 3 = fast to center and zooming away
	elseif levelStage == 3 then
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if timePassed >= 1.0 then
				setNewZoomLevelTarget( 0, 17.0, 1.2, 1.0, 3 )
				timePassed = 1.0
			end
		-- slowly zooming in
		elseif timePassed <= 2.5 then
			timePassed = timePassed + dt
			if timePassed > 2.5 then
				setNewZoomLevelTarget( 0, 17.0, 1.3, 60, 0 )
			end
		end
	
	-- zooming back to the starting pos AFTER the game has been finished (either lost or won)
	elseif levelStage == 4 then
		if timePassed <= 1.0 then
			timePassed = timePassed + dt
			if timePassed > 1.0 then
				if victoryAchieved then 
					showLevelCleared()
				else
					showLevelFailed()
				end
			end
		end
	end
	
	-- spawn enemies
	if levelStage < 4 and gameLost == false then
		enemyIntervalTimeLeft = enemyIntervalTimeLeft - dt
		if enemyIntervalTimeLeft < 0 then
			-- reset timer
			--enemyInterval = enemyInterval * 0.99
			enemyIntervalTimeLeft = enemyInterval
			-- spawn new enemy from left side
			if levelStage == 0 and enemiesSpawned < 90 then
				e1 = math.random( 0, 2 )
				e2 = ( e1 + 1 ) % 3
				e3 = ( e2 + 1 ) % 3
				createNewEnemyUnit( 1, e1, 0 )
				createNewEnemyUnit( math.random( 1, 2 ), e2, 0 )
				createNewEnemyUnit( 4, e3, 0 )
				enemiesSpawned = enemiesSpawned + 3
			-- moving center, spawn enemies from left
			elseif levelStage == 1 and enemiesSpawned < 150 then
				e1 = math.random( 0, 2 )
				e2 = ( e1 + 1 ) % 3
				e3 = ( e2 + 1 ) % 3
				createNewEnemyUnit( 1, e1, 0 )
				createNewEnemyUnit( math.random( 1, 2 ), e2, 0 )
				createNewEnemyUnit( 4, e3, 0 )
				enemiesSpawned = enemiesSpawned + 3
			-- already at center, spawn enemies from both sides
			elseif levelStage == 1 and enemiesSpawned < 250 then
				e1 = math.random( 1, 3 )
				if e1 == 3 then e1 = 4 end
				e2 = math.random( 1, 3 )
				if e2 == 3 then e2 = 4 end
				createNewEnemyUnit( e2, math.random( 0, 2 ), 0 )
				createNewEnemyUnit( e1, math.random( 0, 2 ), 1 )
				createNewEnemyUnit( 5, math.random( 0, 2 ), 1 )
				createNewEnemyUnit( 5, math.random( 0, 2 ), 0 ) 
				enemiesSpawned = enemiesSpawned + 4
			-- moving to the right, only spawn there
			elseif levelStage == 2 and enemiesSpawned < 340 then
				e1 = math.random( 0, 2 )
				e2 = ( e1 + 1 ) % 3
				e3 = ( e2 + 1 ) % 3
				createNewEnemyUnit( 1, e1, 1 )
				createNewEnemyUnit( math.random( 1, 2 ), e2, 1 )
				createNewEnemyUnit( 4, e3, 1 )
				enemiesSpawned = enemiesSpawned + 3
			-- at the center, spawn mass of enemies
			elseif levelStage == 3 and enemiesSpawned < 440 then
				e1 = math.random( 1, 3 )
				if e1 == 3 then e1 = 4 end
				e2 = math.random( 1, 3 )
				if e2 == 3 then e2 = 4 end
				createNewEnemyUnit( e2, math.random( 0, 2 ), 0 )
				createNewEnemyUnit( e1, math.random( 0, 2 ), 1 )
				createNewEnemyUnit( 4, math.random( 0, 2 ), math.random( 0, 1 ) )
				createNewEnemyUnit( 5, math.random( 0, 2 ), 1 )
				createNewEnemyUnit( 5, math.random( 0, 2 ), 0 ) 
				enemiesSpawned = enemiesSpawned + 5
			end
		end
	end
	
	-- advance in stages
	if levelStage == 0 and enemiesSpawned >= 90 then
		levelStage = 1
		timePassed = 0
	elseif levelStage == 1 and enemiesSpawned >= 180 and enemiesLeftOnField() == 0 then
		levelStage = 2
		timePassed = 0
		setSpeedMultipliers( 1.0, 0.86, 0.86, 1, 1, 1 )
	elseif levelStage == 2 and enemiesSpawned >= 260 and enemiesLeftOnField() == 0 then
		levelStage = 3
		timePassed = 0
		setSpeedMultipliers( 1.0, 0.87, 0.87, 1, 1, 1 )
	-- check for victory conditions
	elseif gameLost == false and victoryAchieved == false and levelStage == 3 and enemiesSpawned >= 440 and enemiesLeftOnField() == 0 then
		victoryAchieved = true
		setNewZoomLevelTarget( 0.0, 15.0, 1.0, 1.0, 3 )
		levelStage = 4
		timePassed = 0
	end
end

-- called when a single enemy reaches the base
function enemyReachedBase( healthLeft )
	enemiesDestroyed = enemiesDestroyed + 1
	if healthLeft == 0 and gameLost == false then
		gameLost = true
		setNewZoomLevelTarget( 0.0, 15.0, 1.0, 1.0, 3 )
		timePassed = 0.0
		levelStage = 4
	end
end

-- called when a single enemy has been destroyed
function enemyDestroyed( enemyType )
	-- increase counter
	enemiesDestroyed = enemiesDestroyed + 1
end

-- called when a single weapon has been launched
function weaponLaunched( weaponType )
	--
end

-- calculates the amount of stars to be awarded for completing this level
function calculateStars( scoreCollected, healthLeft )
	if scoreCollected >= ( enemiesSpawned * maxPointsPerEnemy() * 0.8 ) and healthLeft > 4 then
		return 3
	elseif scoreCollected >= ( enemiesSpawned * maxPointsPerEnemy() * 0.5 ) and healthLeft > 2 then
		return 2
	else
		return 1
	end
end

-- for reporting main engine the amount of enemies destroyed during this level
function enemiesDestroyedInLevel()
	return enemiesDestroyed
end

-- for reporting main engine how many enemies there are to come during this level
function enemiesTotalInLevel()
	return 440
end
