-- Zombie Smasher 2
-- Hyperkani Oy 2012
-- Tero Pyylampi
--

-- Campaign 1 Level 8 (camera and lots of fast enemies)

-- for tracking the tutorial states
local timePassed = 0.0
-- enemy data
local enemiesDestroyed = 0
local enemiesSpawned = 0
-- enemy interval
local enemyInterval = 1.25
local enemyIntervalTimeLeft = 4.5
-- victory flag
local victoryAchieved = false
-- game lost flag
local gameLost = false
-- level stages, 0 = center zooming in, 1 = fast movement to left corner zooming in, 2 = fast movement to right corner zooming in, 3 = left corner again, 4 = middle again, 5 = end
local levelStage = 0

-- initialize function which will be called after the game has been initialized
function init()
	-- lock camera
	setCameraLockState( true )
	-- set enemy speeds
	setSpeedMultipliers( 0.5, 1.30, 1.30, 1, 1.25, 1.20 )
end

-- update function which will be called constantly from the main game engine
function update( dt )
	-- starting stage
	if levelStage == 0 then
		-- welcome string after 1 seconds
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if timePassed >= 1.0 then
				showString( "Level 18", 2.5, 0.5, 0.5, 0.65, 34 )
				timePassed = 1.0
			end
		-- initial zoom to the center
		elseif timePassed < 3.0 then
			timePassed = timePassed + dt
			if timePassed >= 3.0 then
				setNewZoomLevelTarget( 0.0, 16.0, 1.1, 0.5, 1 )
				timePassed = 3.0
			end
		elseif timePassed < 3.5 then
			timePassed = timePassed + dt
			if( timePassed >= 3.5 ) then
				setNewZoomLevelTarget( 0.0, 17.0, 1.2, 0.5, 2 )
				timePassed = 3.5
			end
		-- slowly zooming in
		elseif timePassed < 4.5 then
			timePassed = timePassed + dt
			if timePassed >= 4.5 then
				setNewZoomLevelTarget( 0.0, 17.0, 1.4, 60, 0 )
				timePassed = 4.5
			end
		end
	
	-- stage 1 = left corner zooming in
	elseif levelStage == 1 then
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if( timePassed >= 1.0 ) then
				setNewZoomLevelTarget( -22.0, 5.0, 2.0, 1.0, 3 )
				timePassed = 1.0
			end
		-- slowly zooming out
		elseif timePassed < 2.5 then
			timePassed = timePassed + dt
			if timePassed >= 2.5 then
				setNewZoomLevelTarget( -22.0, 5.0, 1.8, 60, 0 )
			end
		end
		
	-- stage 2 = right corner zooming in
	elseif levelStage == 2 then
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if( timePassed >= 1.0 ) then
				setNewZoomLevelTarget( 22.0, 5.0, 2.0, 1.0, 3 )
				timePassed = 1.0
			end
		-- slowly zooming out
		elseif timePassed < 2.5 then
			timePassed = timePassed + dt
			if timePassed >= 2.5 then
				setNewZoomLevelTarget( 22.0, 5.0, 1.8, 60, 0 )
			end
		end
		
	-- stage 3 = left again
	elseif levelStage == 3 then
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if timePassed >= 1.0 then
				setNewZoomLevelTarget( -22.0, 5.0, 2.0, 1.0, 3 )
				timePassed = 1.0
			end
		-- slowly zooming away
		elseif timePassed < 2.5 then
			timePassed = timePassed + dt
			if timePassed >= 2.5 then
				setNewZoomLevelTarget( -22.0, 5.0, 1.8, 60, 0 )
			end
		end
		
	-- stage 4 = middle again
	elseif levelStage == 4 then
		if timePassed < 1.0 then
			timePassed = timePassed + dt
			if timePassed >= 1.0 then
				setNewZoomLevelTarget( 0.0, 17.0, 1.2, 1.0, 3 )
				timePassed = 1.0
			end
		-- slowly zooming in
		elseif timePassed < 2.5 then
			timePassed = timePassed + dt
			if timePassed >= 2.5 then
				setNewZoomLevelTarget( 0, 17.0, 1.4, 60, 0 )
				timePassed = 2.5
			end
		end
	
	-- zooming back to the starting pos AFTER the game has been finished (either lost or won)
	elseif levelStage == 5 then
		if timePassed <= 1.0 then
			timePassed = timePassed + dt
			if timePassed > 1.0 then
				if victoryAchieved then 
					showLevelCleared()
				else
					showLevelFailed()
				end
			end
		end
	end
	
	-- spawn enemies
	if levelStage < 5 and gameLost == false then
		enemyIntervalTimeLeft = enemyIntervalTimeLeft - dt
		if enemyIntervalTimeLeft < 0 then
			-- reset timer
			enemyIntervalTimeLeft = enemyInterval
			-- spawn new enemy from both sides
			if levelStage == 0 and enemiesSpawned < 120 then
				e1 = math.random( 1, 3 )
				if e1 == 3 then e1 = 4 end
				e2 = math.random( 1, 3 )
				if e2 == 3 then e2 = 4 end
				createNewEnemyUnit( e2, math.random( 0, 2 ), 0 )
				createNewEnemyUnit( e1, math.random( 0, 2 ), 1 )
				createNewEnemyUnit( 5, math.random( 0, 2 ), 1 )
				createNewEnemyUnit( 5, math.random( 0, 2 ), 0 ) 
				enemiesSpawned = enemiesSpawned + 4
			-- left corner
			elseif levelStage == 1 and enemiesSpawned < 220 then
				e1 = math.random( 0, 2 )
				e2 = ( e1 + 1 ) % 3
				e3 = ( e2 + 1 ) % 3
				createNewEnemyUnit( math.random( 1, 2 ), e1, 0 )
				createNewEnemyUnit( math.random( 1, 2 ), e2, 0 )
				createNewEnemyUnit( 1, e3, 0 )
				createNewEnemyUnit( 4, math.random( 0, 2 ), 0 )
				enemiesSpawned = enemiesSpawned + 4	
			-- right corner
			elseif levelStage == 2 and enemiesSpawned < 320 then
				e1 = math.random( 0, 2 )
				e2 = ( e1 + 1 ) % 3
				e3 = ( e2 + 1 ) % 3
				createNewEnemyUnit( math.random( 1, 2 ), e1, 1 )
				createNewEnemyUnit( math.random( 1, 2 ), e2, 1 )
				createNewEnemyUnit( 1, e3, 1 )
				createNewEnemyUnit( 4, math.random( 0, 2 ), 1 )
				enemiesSpawned = enemiesSpawned + 4	
			-- back to the left corner
			elseif levelStage == 3 and enemiesSpawned < 420 then
				e1 = math.random( 0, 2 )
				e2 = ( e1 + 1 ) % 3
				e3 = ( e2 + 1 ) % 3
				createNewEnemyUnit( math.random( 1, 2 ), e1, 0 )
				createNewEnemyUnit( math.random( 1, 2 ), e2, 0 )
				createNewEnemyUnit( 1, e3, 0 )
				createNewEnemyUnit( 4, math.random( 0, 2 ), 0 )
				enemiesSpawned = enemiesSpawned + 4	
			-- center position again
			elseif levelStage == 4 and enemiesSpawned < 540 then
				e1 = math.random( 0, 2 )
				e2 = ( e1 + 1 ) % 3
				e3 = math.random( 0, 2 )
				e4 = ( e3 + 1 ) % 3
				createNewEnemyUnit( math.random( 1, 2 ), e1, math.random( 0, 1 ) )
				createNewEnemyUnit( 4, e2, math.random( 0, 1 ) )
				createNewEnemyUnit( math.random( 1, 2 ), e3, math.random( 0, 1 ) )
				createNewEnemyUnit( 5, math.random( 0, 2 ), math.random( 0, 1 ) )  
				enemiesSpawned = enemiesSpawned + 4
			end
		end
	end
	
	-- advance in stages
	if levelStage == 0 and enemiesSpawned >= 120 and enemiesLeftOnField() == 0 then
		levelStage = 1
		timePassed = 0
	elseif levelStage == 1 and enemiesSpawned >= 220 and enemiesLeftOnField() == 0 then
		levelStage = 2
		timePassed = 0
	elseif levelStage == 2 and enemiesSpawned >= 320 and enemiesLeftOnField() == 0 then
		levelStage = 3
		timePassed = 0
	elseif levelStage == 3 and enemiesSpawned >= 420 and enemiesLeftOnField() == 0 then
		levelStage = 4
		timePassed = 0
	-- check for victory conditions
	elseif gameLost == false and victoryAchieved == false and levelStage == 4 and enemiesSpawned >= 540 and enemiesLeftOnField() == 0 then
		victoryAchieved = true
		setNewZoomLevelTarget( 0.0, 15.0, 1.0, 1.0, 3 )
		levelStage = 5
		timePassed = 0
	end
end

-- called when a single enemy reaches the base
function enemyReachedBase( healthLeft )
	enemiesDestroyed = enemiesDestroyed + 1
	if healthLeft == 0 and gameLost == false then
		gameLost = true
		setNewZoomLevelTarget( 0.0, 15.0, 1.0, 1.0, 3 )
		timePassed = 0.0
		levelStage = 5
	end
end

-- called when a single enemy has been destroyed
function enemyDestroyed( enemyType )
	-- increase counter
	enemiesDestroyed = enemiesDestroyed + 1
end

-- called when a single weapon has been launched
function weaponLaunched( weaponType )
	--
end

-- calculates the amount of stars to be awarded for completing this level
function calculateStars( scoreCollected, healthLeft )
	if scoreCollected >= ( enemiesSpawned * maxPointsPerEnemy() * 0.82 ) and healthLeft > 4 then
		return 3
	elseif scoreCollected >= ( enemiesSpawned * maxPointsPerEnemy() * 0.53 ) and healthLeft > 2 then
		return 2
	else
		return 1
	end
end

-- for reporting main engine the amount of enemies destroyed during this level
function enemiesDestroyedInLevel()
	return enemiesDestroyed
end

-- for reporting main engine how many enemies there are to come during this level
function enemiesTotalInLevel()
	return 540
end
