-- Zombie Smasher 2
-- Hyperkani Oy 2012
-- Tero Pyylampi
--

-- Campaign 2 Level 9 (free camera and lots and lots of all enemies)

-- for tracking the tutorial states
local timePassed = 0.0
-- enemy data
local enemiesDestroyed = 0
local enemiesSpawned = 0
-- enemy interval
local enemyInterval = 1.2
local enemyIntervalTimeLeft = 2.3
-- victory flag
local victoryAchieved = false
-- game lost flag
local gameLost = false
-- level stages, 0 = initial stage and complete level, 1 = zooming out as the player has lost or won the game
local levelStage = 0

-- initialize function which will be called after the game has been initialized
function init()
	-- lock camera
	setCameraLockState( true )
	-- set enemy speeds
	setSpeedMultipliers( 0.5, 1.35, 1.30, 1, 1.35, 1.2 )
end

-- update function which will be called constantly from the main game engine
function update( dt )
	-- starting stage
	if levelStage == 0 then
		-- welcome string after 1.5 seconds
		if timePassed <= 1.0 then
			timePassed = timePassed + dt
			if timePassed > 1.0 then
				showString( "Level 19", 2.5, 0.5, 0.5, 0.65, 34 )
				--showString( "(Free camera)", 2.5, 0.5, 0.5, 0.5, 26 )
			end
		-- initial zoom to the center and then releasing the camera
		elseif timePassed <= 3.0 then
			timePassed = timePassed + dt
			if timePassed > 3.0 then
				setNewZoomLevelTarget( 0.0, 16.0, 1.3, 1.0, 3 )
			end
		elseif levelStage == 0 and timePassed <= 4.0 then
			timePassed = timePassed + dt
		-- camera playing a bit
		elseif levelStage == 0 and timePassed > 4.0 then
			timePassed = timePassed + dt
			if( timePassed >= 34.0 ) then
				timePassed = 5.0
				setNewZoomLevelTarget( -3.0 + math.random( 0, 6 ), 14.0 + math.random( 0, 3 ), 1.1 + ( math.random( 0, 4 ) / 10.0 ), 30.0, 0 )
			end
		end
	
	-- showing the end string after the zoom out event (either lost or won)
	elseif levelStage == 1 then
		if timePassed <= 1.0 then
			timePassed = timePassed + dt
			if timePassed > 1.0 then
				if victoryAchieved then 
					showLevelCleared()
				else
					showLevelFailed()
				end
			end
		end
	end
	
	-- spawn enemies
	if levelStage == 0 and gameLost == false then
		enemyIntervalTimeLeft = enemyIntervalTimeLeft - dt
		if enemyIntervalTimeLeft < 0 then
			-- reset timer
			enemyIntervalTimeLeft = enemyInterval
			-- possibly spawn new enemy
			if enemiesSpawned < 100 then
				createNewEnemyUnit( 4, math.random( 0, 2 ), math.random( 0, 1 ) ) 
				createNewEnemyUnit( math.random( 1, 2 ), math.random( 0, 2 ), math.random( 0, 1 ) ) 
				createNewEnemyUnit( 5, math.random( 0, 2 ), 0 ) 
				createNewEnemyUnit( 5, math.random( 0, 2 ), 1 ) 
				enemiesSpawned = enemiesSpawned + 4
				if enemiesSpawned == 100 then
					enemyIntervalTimeLeft = enemyIntervalTimeLeft * 3
				end
			elseif enemiesSpawned < 190 then
				e1 = math.random( 0, 2 )
				e2 = ( e1 + 1 ) % 3
				e3 = ( e2 + 1 ) % 3
				createNewEnemyUnit( math.random( 1, 2 ), e1, 0 ) 
				createNewEnemyUnit( math.random( 1, 2 ), e2, 0 ) 
				createNewEnemyUnit( 4, e3, 0 ) 
				enemiesSpawned = enemiesSpawned + 3
				if enemiesSpawned == 190 then
					enemyIntervalTimeLeft = enemyIntervalTimeLeft * 4
				end
			elseif enemiesSpawned < 280 then
				e1 = math.random( 0, 2 )
				e2 = ( e1 + 1 ) % 3
				e3 = ( e2 + 1 ) % 3
				createNewEnemyUnit( 4, e1, 1 ) 
				createNewEnemyUnit( 1, e2, 1 ) 
				createNewEnemyUnit( 4, e3, 1 )  
				enemiesSpawned = enemiesSpawned + 3
				if enemiesSpawned == 280 then
					enemyIntervalTimeLeft = enemyIntervalTimeLeft * 3
					enemyInterval = enemyInterval - 0.2
				end
			elseif enemiesSpawned <= 400 then
				createNewEnemyUnit( 4, math.random( 0, 2 ), math.random( 0, 1 ) ) 
				createNewEnemyUnit( 1, math.random( 0, 2 ), math.random( 0, 1 ) ) 
				createNewEnemyUnit( 5, math.random( 0, 2 ), 0 ) 
				createNewEnemyUnit( 5, math.random( 0, 2 ), 1 ) 
				enemiesSpawned = enemiesSpawned + 4
			end
		end
	end
	
	-- check for victory conditions
	if gameLost == false and victoryAchieved == false and enemiesSpawned >= 400 and enemiesLeftOnField() == 0 then
		victoryAchieved = true
		setNewZoomLevelTarget( 0.0, 15.0, 1.0, 1.0, 3 )
		levelStage = 1
		timePassed = 0
		setCameraLockState( true )
	end
end

-- called when a single enemy reaches the base
function enemyReachedBase( healthLeft )
	enemiesDestroyed = enemiesDestroyed + 1
	if healthLeft == 0 and gameLost == false then
		gameLost = true
		setNewZoomLevelTarget( 0.0, 15.0, 1.0, 1.0, 3 )
		setCameraLockState( true )
		timePassed = 0.0
		levelStage = 1
	end
end

-- called when a single enemy has been destroyed
function enemyDestroyed( enemyType )
	-- increase counter
	enemiesDestroyed = enemiesDestroyed + 1
end

-- called when a single weapon has been launched
function weaponLaunched( weaponType )
	--
end

-- calculates the amount of stars to be awarded for completing this level
function calculateStars( scoreCollected, healthLeft )
	if scoreCollected >= ( enemiesSpawned * maxPointsPerEnemy() * 0.84 ) and healthLeft > 4 then
		return 3
	elseif scoreCollected >= ( enemiesSpawned * maxPointsPerEnemy() * 0.54 ) and healthLeft > 2 then
		return 2
	else
		return 1
	end
end

-- for reporting main engine the amount of enemies destroyed during this level
function enemiesDestroyedInLevel()
	return enemiesDestroyed
end

-- for reporting main engine how many enemies there are to come during this level
function enemiesTotalInLevel()
	return 400
end
