-- Zombie Smasher 2
-- Hyperkani Oy 2012
-- Tero Pyylampi
--

-- Campaign 1 Level 2 (small playing with camera)

-- for tracking the tutorial states
local timePassed = 0.0
-- enemy data
local enemiesDestroyed = 0
local enemiesSpawned = 0
-- enemy interval
local enemyInterval = 1.2
local enemyIntervalTimeLeft = 1.5
-- victory flag
local victoryAchieved = false
-- game lost flag
local gameLost = false
-- level stages, 0 = lower right corner, 1 = lower left corner, 2 = center, 3 = zooming away
local levelStage = 0

-- initialize function which will be called after the game has been initialized
function init()
	-- lock camera
	setCameraLockState( true )
	-- set enemy speeds
	setSpeedMultipliers( 0.5, 0.7, 1.0, 1, 1, 1 )
end

-- update function which will be called constantly from the main game engine
function update( dt )
	-- starting stage
	if levelStage == 0 then
		-- welcome string after 1.5 seconds
		if timePassed <= 1.0 then
			timePassed = timePassed + dt
			if timePassed > 1.0 then
				showString( "Level 2", 2.5, 0.5, 0.5, 0.65, 34 )
			end
		-- initial zoom to the lower-right corner
		elseif timePassed <= 3.0 then
			timePassed = timePassed + dt
			if timePassed > 3.0 then
				setNewZoomLevelTarget( 11.0, 10.0, 1.45, 0.5, 1 )
			end
		elseif timePassed <= 3.5 then
			timePassed = timePassed + dt
			if( timePassed > 3.5 ) then
				setNewZoomLevelTarget( 22.0, 5.0, 1.9, 0.5, 2 )
			end
		elseif timePassed <= 4.5 then
			timePassed = timePassed + dt
			if timePassed > 4.5 then
				setNewZoomLevelTarget( 22.0, 5.0, 1.7, 90, 0 )
			end
		end
	
	-- stage 1 = move to the left corner
	elseif levelStage == 1 then
		if timePassed <= 1.0 then
			timePassed = timePassed + dt
			if timePassed > 1.0 then
				setNewZoomLevelTarget( 0.0, 14.0, 1.6, 0.5, 1 )
			end
		elseif timePassed <= 1.5 then
			timePassed = timePassed + dt
			if( timePassed > 1.5 ) then
				setNewZoomLevelTarget( -22.0, 5.0, 1.9, 0.5, 2 )
			end
		elseif timePassed <= 2.5 then
			timePassed = timePassed + dt
			if timePassed > 2.5 then
				setNewZoomLevelTarget( -22.0, 5.0, 1.7, 90, 0 )
			end
		end

	-- stage 2 = move to the center position
	elseif levelStage == 2 then
		if timePassed <= 1.0 then
			timePassed = timePassed + dt
			if timePassed > 1.0 then
				setNewZoomLevelTarget( -9.0, 11.5, 1.55, 0.3, 1 )
			end
		elseif timePassed <= 1.3 then
			timePassed = timePassed + dt
			if( timePassed > 1.3 ) then
				setNewZoomLevelTarget( 0.0, 16.0, 1.4, 0.3, 2 )
			end
		elseif timePassed <= 2.1 then
			timePassed = timePassed + dt
			if( timePassed > 2.1 ) then
				setNewZoomLevelTarget( 0.0, 16.0, 1.3, 90, 0 )
			end
		end
	
	-- zooming back to the starting pos AFTER the game has been finished (either lost or won)
	elseif levelStage == 3 then
		if timePassed <= 1.0 then
			timePassed = timePassed + dt
			if timePassed > 1.0 then
				if victoryAchieved then 
					showLevelCleared()
				else
					showLevelFailed()
				end
			end
		end
	end
	
	-- spawn enemies
	if levelStage ~= 3 and gameLost == false then
		enemyIntervalTimeLeft = enemyIntervalTimeLeft - dt
		if enemyIntervalTimeLeft < 0 then
			-- reset timer
			enemyInterval = enemyInterval * 0.99
			enemyIntervalTimeLeft = enemyInterval
			-- spawn new enemy
			if levelStage == 0 and enemiesSpawned < 40 then
				createNewEnemyUnit( 1, math.random( 0, 2 ), 1 )
				enemiesSpawned = enemiesSpawned + 1
			elseif levelStage == 1 and enemiesSpawned < 100 then
				createNewEnemyUnit( 1, math.random( 0, 2 ), 0 ) 
				enemiesSpawned = enemiesSpawned + 1
			elseif levelStage == 2 and enemiesSpawned < 160 then
				-- fixed time
				enemyIntervalTimeLeft = 2.4
				if enemiesSpawned == 74 then
					createNewEnemyUnit( 1, 0, 0 )
					createNewEnemyUnit( 1, 1, 0 ) 
					createNewEnemyUnit( 1, 2, 0 ) 
					createNewEnemyUnit( 1, 0, 1 ) 
					createNewEnemyUnit( 1, 1, 1 ) 
					createNewEnemyUnit( 1, 2, 1 ) 
					enemiesSpawned = enemiesSpawned + 6
				else
					createNewEnemyUnit( 1, math.random( 0, 2 ), 0 ) 
					createNewEnemyUnit( 1, math.random( 0, 2 ), 1 ) 
					enemiesSpawned = enemiesSpawned + 2
				end
			end
		end
	end
	
	-- advance in stages
	if levelStage == 0 and enemiesSpawned >= 40 and enemiesLeftOnField() == 0 then
		levelStage = 1
		timePassed = 0
	elseif levelStage == 1 and enemiesSpawned >= 100 and enemiesLeftOnField() == 0 then
		levelStage = 2
		timePassed = 0
	-- check for victory conditions
	elseif gameLost == false and victoryAchieved == false and levelStage == 2 and enemiesSpawned >= 160 and enemiesLeftOnField() == 0 then
		victoryAchieved = true
		setNewZoomLevelTarget( 0.0, 15.0, 1.0, 1.0, 3 )
		levelStage = 3
		timePassed = 0
	end
end

-- called when a single enemy reaches the base
function enemyReachedBase( healthLeft )
	enemiesDestroyed = enemiesDestroyed + 1
	if healthLeft == 0 and gameLost == false then
		gameLost = true
		setNewZoomLevelTarget( 0.0, 15.0, 1.0, 1.0, 3 )
		timePassed = 0.0
		levelStage = 3
	end
end

-- called when a single enemy has been destroyed
function enemyDestroyed( enemyType )
	-- increase counter
	enemiesDestroyed = enemiesDestroyed + 1
	-- check for empty field
	if enemiesLeftOnField() == 0 then
		enemyIntervalTimeLeft = 0
	end
end

-- called when a single weapon has been launched
function weaponLaunched( weaponType )
	--
end

-- calculates the amount of stars to be awarded for completing this level
function calculateStars( scoreCollected, healthLeft )
	if scoreCollected >= ( enemiesSpawned * maxPointsPerEnemy() * 0.7 ) and healthLeft > 3 then
		return 3
	elseif scoreCollected >= ( enemiesSpawned * maxPointsPerEnemy() * 0.35 ) and healthLeft > 2 then
		return 2
	else
		return 1
	end
end

-- for reporting main engine the amount of enemies destroyed during this level
function enemiesDestroyedInLevel()
	return enemiesDestroyed
end

-- for reporting main engine how many enemies there are to come during this level
function enemiesTotalInLevel()
	return 160
end
