-- Zombie Smasher 2
-- Hyperkani Oy 2012
-- Tero Pyylampi
--

-- Campaign 1 Level 8 (camera and lots of fast enemies)

-- for tracking the tutorial states
local timePassed = 0.0
-- enemy data
local enemiesDestroyed = 0
local enemiesSpawned = 0
-- enemy interval
local enemyInterval = 0.9
local enemyIntervalTimeLeft = 0
-- victory flag
local victoryAchieved = false
-- game lost flag
local gameLost = false
-- level stages, 0 = center zooming in, 1 = fast movement to left corner zooming in, 2 = fast movement to right corner zooming in, 3 = left corner again, 4 = middle again, 5 = end
local levelStage = 0

-- initialize function which will be called after the game has been initialized
function init()
	-- lock camera
	setCameraLockState( true )
	-- set enemy speeds
	setSpeedMultipliers( 1.0, 0.95, 0.95, 1, 1, 1 )
end

-- update function which will be called constantly from the main game engine
function update( dt )
	-- starting stage
	if levelStage == 0 then
		-- welcome string after 1 seconds
		if timePassed <= 1.0 then
			timePassed = timePassed + dt
			if timePassed > 1.0 then
				showString( "Level 8", 2.5, 0.5, 0.5, 0.65, 34 )
			end
		-- initial zoom to the center
		elseif timePassed <= 3.0 then
			timePassed = timePassed + dt
			if timePassed > 3.0 then
				setNewZoomLevelTarget( 0.0, 16.0, 1.2, 0.5, 1 )
			end
		elseif timePassed <= 3.5 then
			timePassed = timePassed + dt
			if( timePassed > 3.5 ) then
				setNewZoomLevelTarget( 0.0, 17.0, 1.4, 0.5, 2 )
			end
		-- slowly zooming in
		elseif timePassed <= 4.5 then
			timePassed = timePassed + dt
			if timePassed > 4.5 then
				setNewZoomLevelTarget( 0.0, 17.0, 1.5, 30, 0 )
			end
		end
	
	-- stage 1 = left corner zooming in
	elseif levelStage == 1 then
		if timePassed <= 1.0 then
			timePassed = timePassed + dt
			if timePassed > 1.0 then
				setNewZoomLevelTarget( -11, 11.0, 1.7, 0.5, 1 )
			end
		elseif timePassed <= 1.5 then
			timePassed = timePassed + dt
			if( timePassed > 1.5 ) then
				setNewZoomLevelTarget( -22.0, 5.0, 1.9, 0.5, 2 )
			end
		-- slowly zooming out
		elseif timePassed <= 4.5 then
			timePassed = timePassed + dt
			if timePassed > 4.5 then
				setNewZoomLevelTarget( -22.0, 5.0, 1.7, 60, 0 )
			end
		end
		
	-- stage 2 = right corner zooming in
	elseif levelStage == 2 then
		if timePassed <= 1.0 then
			timePassed = timePassed + dt
			if timePassed > 1.0 then
				setNewZoomLevelTarget( 0, 12.0, 1.7, 0.5, 1 )
			end
		elseif timePassed <= 1.5 then
			timePassed = timePassed + dt
			if( timePassed > 1.5 ) then
				setNewZoomLevelTarget( 22.0, 5.0, 1.9, 0.5, 2 )
			end
		-- slowly zooming out
		elseif timePassed <= 4.5 then
			timePassed = timePassed + dt
			if timePassed > 4.5 then
				setNewZoomLevelTarget( 22.0, 5.0, 1.7, 60, 0 )
			end
		end
		
	-- stage 3 = left again
	elseif levelStage == 3 then
		if timePassed <= 1.0 then
			timePassed = timePassed + dt
			if timePassed > 1.0 then
				setNewZoomLevelTarget( 0, 12.0, 1.7, 0.5, 1 )
			end
		elseif timePassed <= 1.5 then
			timePassed = timePassed + dt
			if( timePassed > 1.5 ) then
				setNewZoomLevelTarget( -22.0, 5.0, 1.9, 0.5, 2 )
			end
		-- slowly zooming away
		elseif timePassed <= 4.5 then
			timePassed = timePassed + dt
			if timePassed > 4.5 then
				setNewZoomLevelTarget( -22.0, 5.0, 1.7, 60, 0 )
			end
		end
		
	-- stage 4 = middle again
	elseif levelStage == 4 then
		if timePassed <= 1.0 then
			timePassed = timePassed + dt
			if timePassed > 1.0 then
				setNewZoomLevelTarget( -11.0, 10.0, 1.55, 0.5, 1 )
			end
		elseif timePassed <= 1.5 then
			timePassed = timePassed + dt
			if( timePassed > 1.5 ) then
				setNewZoomLevelTarget( 0, 16.0, 1.4, 0.5, 2 )
			end
		-- slowly zooming in
		elseif timePassed <= 4.5 then
			timePassed = timePassed + dt
			if timePassed > 4.5 then
				setNewZoomLevelTarget( 0, 15.0, 1.3, 60, 0 )
			end
		end
	
	-- zooming back to the starting pos AFTER the game has been finished (either lost or won)
	elseif levelStage == 5 then
		if timePassed <= 1.0 then
			timePassed = timePassed + dt
			if timePassed > 1.0 then
				if victoryAchieved then 
					showLevelCleared()
				else
					showLevelFailed()
				end
			end
		end
	end
	
	-- spawn enemies
	if levelStage < 5 and gameLost == false then
		enemyIntervalTimeLeft = enemyIntervalTimeLeft - dt
		if enemyIntervalTimeLeft < 0 then
			-- reset timer
			enemyIntervalTimeLeft = enemyInterval
			-- spawn new enemy from both sides
			if levelStage == 0 and enemiesSpawned < 112 then
				e1 = math.random( 0, 2 )
				e2 = ( e1 + 1 ) % 3
				e3 = ( e2 + 1 ) % 3
				createNewEnemyUnit( math.random( 1, 2 ), e1, math.random( 0, 1 ) )
				createNewEnemyUnit( 1, e2, math.random( 0, 1 ) )
				createNewEnemyUnit( math.random( 1, 2 ), e3, math.random( 0, 1 ) )
				enemiesSpawned = enemiesSpawned + 3
			-- left corner
			elseif levelStage == 1 and enemiesSpawned < 172 then
				e1 = math.random( 0, 2 )
				e2 = ( e1 + 1 ) % 3
				e3 = ( e2 + 1 ) % 3
				createNewEnemyUnit( math.random( 1, 2 ), e1, 0 )
				createNewEnemyUnit( math.random( 1, 2 ), e2, 0 )
				createNewEnemyUnit( 1, e3, 0 )
				enemiesSpawned = enemiesSpawned + 3
			-- right corner
			elseif levelStage == 2 and enemiesSpawned < 268 then
				e1 = math.random( 0, 2 )
				e2 = ( e1 + 1 ) % 3
				e3 = ( e2 + 1 ) % 3
				createNewEnemyUnit( 1, e1, 1 )
				createNewEnemyUnit( 2, e2, 1 )
				createNewEnemyUnit( math.random( 1, 2 ), e3, 1 )
				enemiesSpawned = enemiesSpawned + 3
			-- back to the left corner
			elseif levelStage == 3 and enemiesSpawned < 340 then
				e1 = math.random( 0, 2 )
				e2 = ( e1 + 1 ) % 3
				e3 = ( e2 + 1 ) % 3
				createNewEnemyUnit( 1, e1, 0 )
				createNewEnemyUnit( 2, e2, 0 )
				createNewEnemyUnit( 2, e3, 0 )
				enemiesSpawned = enemiesSpawned + 3
			-- center position again
			elseif levelStage == 4 and enemiesSpawned < 500 then
				createNewEnemyUnit( 1, math.random( 0, 2 ), 0 ) 
				createNewEnemyUnit( 1, math.random( 0, 2 ), 0 ) 
				createNewEnemyUnit( 1, math.random( 0, 2 ), 1 )
				createNewEnemyUnit( 2, math.random( 0, 2 ), 1 )
				createNewEnemyUnit( 2, math.random( 0, 2 ), math.random( 0, 1 ) ) 
				enemiesSpawned = enemiesSpawned + 5
			end
		end
	end
	
	-- advance in stages
	if levelStage == 0 and enemiesSpawned >= 112 and enemiesLeftOnField() == 0 then
		levelStage = 1
		timePassed = 0
		enemyInterval = 2.2
	elseif levelStage == 1 and enemiesSpawned >= 172 and enemiesLeftOnField() == 0 then
		levelStage = 2
		timePassed = 0
		setSpeedMultipliers( 1.0, 0.93, 0.93, 1, 1, 1 )
		enemyInterval = 2.1
	elseif levelStage == 2 and enemiesSpawned >= 268 and enemiesLeftOnField() == 0 then
		levelStage = 3
		timePassed = 0
		setSpeedMultipliers( 1.0, 0.94, 0.94, 1, 1, 1 )
		enemyInterval = 2.0
	elseif levelStage == 3 and enemiesSpawned >= 340 and enemiesLeftOnField() == 0 then
		levelStage = 4
		timePassed = 0
		setSpeedMultipliers( 1.0, 0.95, 0.95, 1, 1, 1 )
		enemyInterval = 1.9
	-- check for victory conditions
	elseif gameLost == false and victoryAchieved == false and levelStage == 4 and enemiesSpawned >= 500 and enemiesLeftOnField() == 0 then
		victoryAchieved = true
		setNewZoomLevelTarget( 0.0, 15.0, 1.0, 1.0, 3 )
		levelStage = 5
		timePassed = 0
	end
end

-- called when a single enemy reaches the base
function enemyReachedBase( healthLeft )
	enemiesDestroyed = enemiesDestroyed + 1
	if healthLeft == 0 and gameLost == false then
		gameLost = true
		setNewZoomLevelTarget( 0.0, 15.0, 1.0, 1.0, 3 )
		timePassed = 0.0
		levelStage = 5
	end
end

-- called when a single enemy has been destroyed
function enemyDestroyed( enemyType )
	-- increase counter
	enemiesDestroyed = enemiesDestroyed + 1
end

-- called when a single weapon has been launched
function weaponLaunched( weaponType )
	--
end

-- calculates the amount of stars to be awarded for completing this level
function calculateStars( scoreCollected, healthLeft )
	if scoreCollected >= ( enemiesSpawned * maxPointsPerEnemy() * 0.82 ) and healthLeft > 4 then
		return 3
	elseif scoreCollected >= ( enemiesSpawned * maxPointsPerEnemy() * 0.53 ) and healthLeft > 2 then
		return 2
	else
		return 1
	end
end

-- for reporting main engine the amount of enemies destroyed during this level
function enemiesDestroyedInLevel()
	return enemiesDestroyed
end

-- for reporting main engine how many enemies there are to come during this level
function enemiesTotalInLevel()
	return 502
end
